require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::InstallPE < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super

    @host = @infra.hosts[data[:host]] if data[:host]
    @stop_on_fail = true

    @log_dir = '/var/log/pe-installer'
    @local_installer = '/opt/puppet/share/installer'
    @local_inst = @local_installer + '/installer'
    @local_ans = @local_installer + '/answers'
    @installer_etc = '/etc/puppetlabs/installer'
  end

  # Call from executor with a block to handle the actions.
  def perform_step(&execution_block)
    mktmp = Installer::Action::Execute.new(:hosts => [@host], :cmd => "mktemp -d -t pe-installer-XXXXXXXX")
    yield mktmp
    unless mktmp.success?
      error(@host.hostname, "Failed to create a temporary directory for the Puppet Enterprise installation files: #{mktmp.results}.")
      return
    end

    # TODO: Some parsing for the tmp dir? Define these better
    tmpdir = mktmp.results[@host.hostname][:stdout].strip
    installer_dir = "#{tmpdir}/install"
    answer_file = "#{tmpdir}/#{@host.hostname}.answers"

    chmodtmp = Installer::Action::Execute.new(:hosts => [@host], :cmd => "chmod 700 '#{tmpdir}'")
    yield chmodtmp
    unless chmodtmp.success?
      error(@host.hostname, "Failed to set permissions on temporary directory for the Puppet Enterprise installation files: #{tmpdir}.")
      return
    end

    # Ensure log dir
    mklogdir = Installer::Action::Execute.new(:hosts => [@host], :sudo => true, :cmd => "mkdir -p #{@log_dir}")
    yield mklogdir
    unless mklogdir.success?
      error(@host.hostname, "Failed to create directory '#{@log_dir}'. #{mklogdir.error_message(@host.hostname)}")
      return
    end

    # Chmod log dir
    logdir_chmod_cmd = "chmod 700 '#{@log_dir}'"
    chmodlogdir = Installer::Action::Execute.new(:hosts => [@host], :sudo => true, :cmd => logdir_chmod_cmd)
    yield chmodlogdir
    unless chmodlogdir.success?
      error(@host.hostname, "Failed to execute '#{logdir_chmod_cmd}'. #{chmodlogdir.error_message(@host.hostname)}")
      return
    end

    # Ensure /etc/puppetlabs/installer dir
    mketcdir = Installer::Action::Execute.new(:hosts => [@host], :sudo => true, :cmd => "mkdir -p #{@installer_etc}/")
    yield mketcdir
    unless mketcdir.success?
      error(@host.hostname, "Failed to create directory '#{@installer_etc}'. #{mketcdir.error_message(@host.hostname)}")
      return
    end

    # Chmod installer dir
    installdir_chmod_cmd = "chmod 700 '#{@installer_etc}'"
    chmodinstalldir = Installer::Action::Execute.new(:hosts => [@host], :sudo => true, :cmd => installdir_chmod_cmd)
    yield chmodinstalldir
    unless chmodinstalldir.success?
      error(@host.hostname, "Failed to execute '#{installdir_chmod_cmd}'. #{chmodinstalldir.error_message(@host.hostname)}")
      return
    end

    # Ensure there is an installer available to copy
    unless File.readable? "#{@local_inst}/puppet-enterprise-installer"
      error("localhost", "The Puppet Enterprise installer was not found in '#{@local_inst}'.")
      return
    end

    # Copy installer files
    installer = Installer::Action::Upload.new(:to_host => @host, :from => @local_inst, :to => installer_dir, :opts => {:recursive => true})
    yield installer
    return unless installer.success?

    # Copy answers file and save a copy locally
    answers = @host.generate_answers
    write_local_answers(@host.hostname, answers)

    answers = Installer::Action::Upload.new(:to_host => @host, :content => answers, :to => answer_file)
    yield answers
    return unless answers.success?
    
    # Run installer with answer file
    install = Installer::Action::Execute.new(:host => @host, :sudo => true, :cmd => "#{installer_dir}/puppet-enterprise-installer -D -a #{answer_file}")
    yield install
    unless install.success?
      error(@host.hostname, "Failed to run PE installer on #{@host.hostname}")
      return
    end

    # Cleanup tmp if install succeeded. If not, we want to keep these files around
    # for troubleshooting purposes.
    cleanup = Installer::Action::Execute.new(:host => @host, :sudo => true, :cmd => "rm -rf '#{tmpdir}'")
    yield cleanup
    unless cleanup.success?
      error(@host.hostname, "Failed to clean up temp directory '#{tmpdir}' on #{@host.hostname}")
      return
    end
  end

  def write_local_answers(hostname, answers)
    Dir.mkdir(@local_ans) unless File.directory? @local_ans
    File.open("#{@local_ans}/#{hostname}.answers", 'w') { |f| f.puts answers }
  end

  def fail_stop?
    true
  end

  def name
    "install_pe_#{@host}"
  end

  def description
    "Installing Puppet Enterprise on #{@host}."
  end
end
